"""
Tests for nap module.

These tests only focus that requests is called properly.
Everything related to HTTP requests should be tested in requests' own tests.
"""

from mock import MagicMock, patch
import unittest
import requests

from nap.url import Url


class TestNap(unittest.TestCase):

    @patch('requests.request')
    def test_join_urls(self, r_request):
        """Test creating new sub-Urls"""
        url = Url('http://domain.com')
        r_request = MagicMock(return_value=None)

        new_url = url.join('/path/a/b')
        new_url.get()
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/path/a/b'
        )

    @patch('requests.request')
    def test_joined_urls_option_passing(self, r_request):
        """Test that original options are correctly passed to joined urls"""
        url = Url(
            'http://domain.com',
            auth=('user', 'pass')
        )
        r_request = MagicMock(return_value=None)

        new_url = url.join('path')
        new_url.get()
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/path',
            auth=('user', 'pass')
        )

    @patch('requests.request')
    def test_join_url_preserves_original_url(self, r_request):
        """Test that original url is not touched when joining urls."""
        url = Url('http://domain.com/')
        r_request = MagicMock(return_value=None)

        new_url = url.join('/path')
        new_url.get()
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/path'
        )

        new_url = url.join('/path/')
        new_url.get()
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/path/'
        )

    def test_requests_raises_error(self):
        """Test that requests properly raises its own errors

        >>> requests.get('/kk')
        requests.exceptions.MissingSchema: Invalid URL u'/kk':
        No schema supplied. Perhaps you meant http:///kk?
        """
        url = Url('')
        self.assertRaises(requests.exceptions.MissingSchema, url.get)

    @patch('requests.request')
    def test_default_parameters(self, r_request):
        """Test default parameter behavior"""
        url = Url('http://domain.com', auth=('user', 'password'))
        r_request = MagicMock(return_value=None)

        # Make sure defaults are passed for each request
        url.get('resource')
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/resource',
            auth=('user', 'password')
        )

        # Make sure single calls can override defaults
        url.get('resource', auth=('defaults', 'overriden'))
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/resource',
            auth=('defaults', 'overriden')
        )

    def test_url_getter_setter(self):
        """Test url attribute"""
        base_url = 'http://domain.com'
        url = Url(base_url)

        self.assertEqual(url.url, base_url)

        # Inline function definition instead of using with keyword to support
        # python 2.6 unittest module
        def test_set(url):
            url.url = 'http://newdomain.com'

        # url attribute should be read-only
        self.assertRaises(AttributeError, test_set, url)

    def test_all_methods(self):
        """Test all HTTP methods"""
        url = Url('http://domain.com')
        self._test_all_methods(url, 'http://domain.com')

        new_url = url.join('path/a/b')
        self._test_all_methods(new_url, 'http://domain.com/path/a/b')

    @patch('requests.request')
    def _test_all_methods(self, url_obj, expected_url, r_request):
        """Test all methods for given url object"""
        r_request = MagicMock(return_value=None)

        methods = ['delete', 'get', 'head', 'patch', 'post', 'put']

        for method in methods:
            getattr(url_obj, method.lower())()

            requests.request.assert_called_with(
                method.upper(),
                expected_url
            )



"""
Tests for nap module.

These tests only focus that requests is called properly.
Everything related to HTTP requests should be tested in requests' own tests.
"""

from mock import MagicMock, patch
import unittest
import requests

from nap.url import Url


class NewUrl(Url):

    def before_request(self, method, request_kwargs):
        request_kwargs['test'] = 'test'
        return request_kwargs

    def after_request(self, response):
        return None

    def default_kwargs(self):
        # Remove all default kwargs
        return {}


class TestNapExtendBehavior(unittest.TestCase):

    @patch('requests.request')
    def test_before_request_and_after_request(self, r_request):
        """Test overriding before_request() and after_request()"""
        url = NewUrl('http://domain.com')
        r_request = MagicMock(return_value=1)

        # Make sure defaults are passed for each request
        response = url.get('resource')
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/resource',
            test='test'
        )

        # Mocker will return 1 to after_request and we have modified it
        # to return just None
        self.assertEquals(
            response,
            None,
            'after_request overriding not working'
        )

    @patch('requests.request')
    def test_default_kwargs(self, r_request):
        """Test overriding default_kwargs()"""
        # We give default arguments, but the overriding implementation
        # of default_kwargs() method should throw them away
        url = NewUrl('http://domain.com', auth=('user', 'pass'))
        r_request = MagicMock(return_value=1)

        # Make sure defaults were removed from kwargs
        url.get('resource')
        requests.request.assert_called_with(
            'GET',
            'http://domain.com/resource',
            # This will still be in kwargs because manipulation
            # of default_kwargs and request_kwargs are separated
            test='test'
        )



"""
Convenient way to request HTTP APIs.

For examples, see http://github.com/kimmobrunfeldt/nap
"""

import sys

_PY3 = sys.version_info >= (3, 0)

# For Python 3 compatibility
if _PY3:
    from urllib.parse import urljoin
else:
    from urlparse import urljoin

import requests


class Url(object):
    """Wrapper class for requests library."""

    def __init__(self, base_url, **default_kwargs):
        """
        * `base_url`
            API's base url. Trailing slash is optional.
            For example `'https://api.github.com'`

        * `**default_kwargs`
            Keyword arguments that will be passed to
            `requests.request` on each request
        """
        self._base_url = base_url
        self._default_kwargs = default_kwargs

    @property
    def url(self):
        """Returns base url"""
        return self._base_url

    def join(self, relative_url):
        """Joins base url with relative_url and returns new Url object
        from the combined url.
        """
        return self._new_url(relative_url)

    # HTTP methods

    def delete(self, *args, **kwargs):
        return self._request('DELETE', *args, **kwargs)

    def get(self, *args, **kwargs):
        return self._request('GET', *args, **kwargs)

    def head(self, *args, **kwargs):
        return self._request('HEAD', *args, **kwargs)

    def patch(self, *args, **kwargs):
        return self._request('PATCH', *args, **kwargs)

    def post(self, *args, **kwargs):
        return self._request('POST', *args, **kwargs)

    def put(self, *args, **kwargs):
        return self._request('PUT', *args, **kwargs)

    # Overridable methods to extend behavior

    def before_request(self, method, request_kwargs):
        """This method can be overridden to customize each request.

        * `method`
            The HTTP method of request in upper case. For example `'GET'`.

        * `request_kwargs`
            Keyword arguments that were passed to the request method.
            This does not contain the default keyword arguments given when
            initializing Api class. For example in `url.get(verify=False)`,
            the value would be `{'verify': False}`.

        This method should return keyword arguments. These returned kwargs
        will be added on top of default request_kwargs given to class.
        The sum of both keyword arguments will be passed to  `requests.request`
        """
        return request_kwargs

    def after_request(self, response):
        """This method can be overridden to add default behavior when response
        is returned. For example if you're working with a JSON API, you can
        return deserialized JSON from this method instead of `Response` object.

        * `response`
            `requests.Response` object returned by *request* function.

        The returned value will be returned to the API method caller:
        `response = api.get('users')`.
        """
        return response

    def default_kwargs(self):
        """This method can be overridden to modify `default_kwargs` given in
        class initialization.

        Returns new default kwargs.
        """
        return self._default_kwargs

    def _request(self, http_method, relative_url='', **kwargs):
        """Does actual HTTP request using requests library."""
        # It could be possible to call api.resource.get('/index')
        # but it would be non-intuitive that the path would resolve
        # to root of domain
        relative_url = self._remove_leading_slash(relative_url)

        # Add default kwargs with possible custom kwargs returned by
        # before_request
        new_kwargs = self.default_kwargs().copy()
        custom_kwargs = self.before_request(
            http_method,
            kwargs.copy()
        )
        new_kwargs.update(custom_kwargs)

        response = requests.request(
            http_method,
            self._join_url(relative_url),
            **new_kwargs
        )

        return self.after_request(response)

    def _join_url(self, relative_url):
        """Joins relative url with base url. Adds trailing slash if needed."""
        joined_url = urljoin(self._base_url, relative_url)
        return joined_url

    def _remove_leading_slash(self, text):
        return text[1:] if text.startswith('/') else text

    def _new_url(self, relative_url):
        """Create new Url which points to new url."""

        return Url(
            urljoin(self._base_url, relative_url),
            **self._default_kwargs
        )



#!/usr/bin/env python

from pip.req import parse_requirements

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt')

# reqs is a list of requirement
# e.g. ['django==1.5.1', 'mezzanine==1.4.6']
reqs = [str(ir.req) for ir in install_reqs]


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


readme = """Read docs from GitHub_

.. _GitHub: https://github.com/kimmobrunfeldt/nap
"""

setup(
    name='nap',
    version='1.0.1',
    description='Convenient way to request HTTP APIs',
    long_description=readme,
    author='Kimmo Brunfeldt',
    author_email='kimmobrunfeldt@gmail.com',
    url='https://github.com/kimmobrunfeldt/nap',
    packages=[
        'nap',
    ],
    package_dir={'nap': 'nap'},
    include_package_data=True,
    install_requires=reqs,
    license='MIT',
    zip_safe=False,
    keywords='nap rest requests http',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)
